use 5.010000;
use strict;
use warnings;

use Test::Whitespaces {

    dirs => [
         'lib',
        't',
    ],

    files => [
        'README',
        'Makefile.PL',
        'Changes',
    ],

};
